# -*- coding: utf-8 -*-
"""
API
===

.. autoclass:: JumboModelAdmin
   :members:


"""

from functools import update_wrapper
from django import forms, VERSION
from django import template
from django.core.urlresolvers import reverse
from django.db import transaction
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.utils import text
from django.contrib.admin import helpers
try:
    from django.contrib.admin.utils import unquote, flatten_fieldsets
except:
    ## Django < 1.7
    from django.contrib.admin.util import unquote, flatten_fieldsets
from django.utils.encoding import force_unicode
from django.shortcuts import render_to_response
from django.http import Http404, HttpResponseRedirect, HttpResponse
from django.utils.html import escape
from django.views.decorators.csrf import csrf_protect
from django.conf import settings
from django.utils.decorators import method_decorator
from django.core.exceptions import PermissionDenied
from django.contrib import messages
from django.conf.urls import url
from ajax_inlines import AjaxParentModelAdmin
from django.contrib import admin

from . import actions

# GET parameter for the URL to return to after change/add views. This lets the
# admin save the state of the changelist page through the change/add page.
#RETURN_GET_PARAM = '_return_to'

class JumboListPerPageForm(forms.Form):
    LIST_PER_PAGE_CHOICES = (
        (0, '---'),
        (10, 10), (25, 25),
        (50, 50), (100, 100),
    )
    choices = getattr(settings, 'LIST_PER_PAGE_CHOICES', LIST_PER_PAGE_CHOICES)
    list_per_page = forms.ChoiceField(label=_('List per page'), choices=choices)


class JumboModelAdmin(AjaxParentModelAdmin):

    class Media:

        js = ['jmb/js/collapsed_stacked_inlines.js']

    #: tabs: show tabs even if they're empty
    tabs_show_empty = True

    #: tabs: show tab lables sticky on top of the page
    tabs_sticky_on_top = False

    no_display_links = True
    list_per_page = 20
    save_on_top = True
    use_fancybox = False
    father_foreignkey_fancybox = None
    list_per_page_form = JumboListPerPageForm
    paginator_on_top = getattr(settings, 'PAGINATOR_ON_TOP', False)
    paginator_on_bottom = getattr(settings, 'PAGINATOR_ON_BUTTOM', True)

    def get_action_form_instance(self, request):
        """
        Return an ``action_form``
        """
        ###########  Django's official code 1.7 -- begin ##############
        # There can be multiple action forms on the page (at the top
        # and bottom of the change list, for example). Get the action
        # whose button was pushed.
        try:
            action_index = int(request.POST.get('index', 0))
        except ValueError:
            action_index = 0

        # Construct the action form.
        data = request.POST.copy()
        data.pop(helpers.ACTION_CHECKBOX_NAME, None)
        data.pop("index", None)

        # Use the action whose button was pushed
        try:
            data.update({'action': data.getlist('action')[action_index]})
        except IndexError:
            # If we didn't get an action from the chosen form that's invalid
            # POST data, so by deleting action it'll fail the validation check
            # below. So no need to do anything here
            pass

        action_form = self.action_form(data, auto_id=None)
        ###########  Django's official code 1.7 -- end ##############
        action_form.fields['action'].choices = self.get_action_choices(request)
        action_form.is_valid()
        return action_form

    def get_list_per_page_form(self, request):
        """
        Returns number of items appear on each paginated admin change list page
        """
        try:
            return self.list_per_page_form
        except:
            return None

    def get_list_display_links(self, request, list_display):
        return ('get_edit_icon',)

    def get_list_per_page(self):
        """
        Set how many items appear on each paginated admin change list page.
        This value is conditioned by settings.LIST_PER_PAGE.
        Example:
            LIST_PER_PAGE = {
                'content_content': 50,
                'content': 10,
                'all': 100,
            }
        """
        if hasattr(settings, 'LIST_PER_PAGE'):
            list_per_page = settings.LIST_PER_PAGE
            app_label = self.model._meta.app_label.lower()
            model_name = self.model._meta.verbose_name_raw.lower()

            key = app_label + '_' + model_name
            if key in list_per_page:
                return list_per_page[key]

            key = app_label
            if key in list_per_page:
                return list_per_page[key]

            key = 'all'
            if key in list_per_page:
                return list_per_page[key]

        return 50

    def get_readonly_fields(self, request, obj=None):
        """Add to whatever fields where defined also
        creator, date_create, last_modifier, date_last_modifier"
        """

        readonly = []
        field_names = [f.name for f in self.model._meta.fields]

        if 'creator' in field_names:
            readonly = readonly + ['creator', ]
        if 'date_create' in field_names:
            readonly = readonly + ['date_create', ]
        if 'last_modifier' in field_names:
            readonly = readonly + ['last_modifier', ]
        if 'date_last_modify' in field_names:
            readonly = readonly + ['date_last_modify', ]
        return self.readonly_fields + tuple(readonly)

    def get_changelist_instance(self, request):
        """Return a chagelist instance suitable to find out the real queryset represented bu result_list
        This function can be used in actions when the queryset is much faster then
        using the list of single ids
        """
        kw = dict(model=self.model, list_display=self.list_display,
                  list_display_links=self.list_display_links, list_filter=self.list_filter,
                  date_hierarchy=self.date_hierarchy, search_fields=self.search_fields, list_select_related=self.list_select_related,
                  list_per_page=self.list_per_page, list_max_show_all=self.list_max_show_all,
                  list_editable=self.list_editable, model_admin=self)
        return self.get_changelist(request)(request, **kw)

    def _getobj(self, request, object_id):
        opts = self.model._meta
        app_label = opts.app_label

        try:
            obj = self.queryset(request).get(pk=unquote(object_id))
        except self.model.DoesNotExist:
            obj = None

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') %
                          {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
        return obj

    def _wrap(self, view):
        def wrapper(*args, **kwargs):
            return self.admin_site.admin_view(view)(*args, **kwargs)
        return update_wrapper(wrapper, view)

    def _view_name(self, name):
        if VERSION[:2] < (1, 6):
            info = self.model._meta.app_label, self.model._meta.module_name, name
        else:
            info = self.model._meta.app_label, self.model._meta.model_name, name
        return '%s_%s_%s' % info

    def has_change_permission(self, request, obj=None):
        has_change_permission = super(JumboModelAdmin, self).has_change_permission(request, obj)
        if VERSION[:2] < (1, 6):
            model_name = self.model._meta.module_name
        else:
            model_name = self.model._meta.model_name

        if obj is None:
            if (
                has_change_permission or
                request.user.has_perm(self.model._meta.app_label + '.' + 'view_%s' % model_name) or
                request.user.has_perm(self.model._meta.app_label + '.' + 'list_%s' % model_name)
            ):
                return True
            return False
        else:
            if hasattr(obj, "can_be_modified"):
                if obj.can_be_modified():
                    return True
                else:
                    return False
            return has_change_permission

    def has_delete_permission(self, request, obj=None):
        has_delete_permission = super(JumboModelAdmin, self).has_delete_permission(request, obj)
        if hasattr(obj, "can_be_delete"):
            if obj.can_be_delete():
                return True
            else:
                return False

        return has_delete_permission

    def get_value_obj(self, obj, attribute):
        value = None
        try:
            value = getattr(obj, attribute)
        except:
            try:
                if hasattr(self, attribute):
                    value = getattr(self, attribute)(obj)
            except Exception, e:
                pass
        return value

    def detail_view(self, request, object_id, **kwargs):
        obj = self._getobj(request, object_id)
        opts = self.model._meta
        verbose_name = opts.verbose_name
        app_label = opts.app_label.lower()
        object_name = opts.object_name.lower()

        detail_fields = []
        d_values = {}
        template_name = kwargs.get('template_name', 'detail.html')

        if hasattr(self, 'detail_display'):
            for f in self.detail_display:
                # Se e' una lista aggiunga tt i suoi campi ai detail_fields
                if isinstance(f, (list, tuple)):
                    detail_fields.append(f)
                    for subf in f:
                        d_values[subf] = self.get_value_obj(obj, subf)
                # Se la funzione e' presente nell'admin la chiamo e aggiungo il
                # valore ai detail fields
                elif hasattr(self, f):
                    detail_fields.append([f])
                    d_values[f] = getattr(self, f)(obj)
                # altrimenti cerco il valore dell'oggetto in maniera normale
                else:
                    detail_fields.append([f])
                    d_values[f] = self.get_value_obj(obj, f)

        context = {
            'title': _('Detail %s') % force_unicode(verbose_name),
            'object_id': object_id,
            'obj': obj,
            'is_popup': '_popup' in request.GET,
            'app_label': app_label,
            'opts': opts,
            'detail_fields': detail_fields,
            'd_values': d_values,
            'has_change_permission': self.has_change_permission(request, obj),
        }
        context_instance = template.RequestContext(request, current_app=self.admin_site.name)

        return render_to_response([
            "admin/%s/%s/%s" % (app_label, object_name, template_name),
            "admin/%s/%s" % (app_label, template_name),
            "admin/%s" % template_name
        ], context, context_instance=context_instance)

    #nuovo nome, retrocompatibilità
    detail = detail_view

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['is_popup'] = request.GET.get("_popup", False)
#        extra_context['action_form'] = self.get_action_form(request)
        extra_context['list_per_page_form'] = self.get_list_per_page_form(request)
        extra_context['paginator_on_top'] = self.paginator_on_top
        extra_context['paginator_on_bottom'] = self.paginator_on_bottom

        self.list_per_page = self.get_list_per_page()
        return super(JumboModelAdmin, self).changelist_view(request, extra_context)

    def get_urls(self):
        urls = super(JumboModelAdmin, self).get_urls()

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        if VERSION[:2] < (1, 6):
            info = self.model._meta.app_label, self.model._meta.module_name
        else:
            info = self.model._meta.app_label, self.model._meta.model_name

        my_urls = [
            url(
                r'^(.+)/detail/$',
                self._wrap(self.detail),
                name=self._view_name('detail')
            ),
        ]
        return my_urls + urls

    def response_action(self, request, queryset):
        """
        Handle an admin action. This is called if a request is POSTed to the
        changelist; it returns an HttpResponse if the action was handled, and
        None otherwise.

        This method just adds/changes the message in case of errors

        """
        response = super(JumboModelAdmin, self).response_action(request, queryset)
        if response:
            return response
        else:
            form = self.get_action_form_instance(request)

            if 'action' in form.errors and len(form.errors) == 1:
                ## This case is already handled by django
                return

            ## We need to remove django's last message generated by response_action
            ## as it states no action was selected storage.used = True doesn't work
            # see
            # http://stackoverflow.com/questions/4083164/django-remove-message-before-they-are-displayed
            storage = messages.get_messages(request)
            for _ in storage:
                pass

            if len(storage._loaded_messages) == 1:
                # it's surely the one generated by django's response_action
                del storage._loaded_messages[0]

            def no_error_class(fname, error_msg):
                return text.mark_safe(text.force_unicode(error_msg.as_text().lstrip('*')))

            msg = "<br>".join(no_error_class(fname, error_msg)
                              for fname, error_msg in form.errors.iteritems())

            self.message_user(request, text.mark_safe(msg), messages.ERROR)
            return None

    def message_user(self, request, message, level=messages.INFO, extra_tags='',
                     fail_silently=False):
        """
        Send a message to the user. The default implementation
        posts a message using the django.contrib.messages backend.

        Exposes almost the same API as messages.add_message(), but accepts the
        positional arguments in a different order to maintain backwards
        compatibility. For convenience, it accepts the `level` argument as
        a string rather than the usual level number.
        """

        if not isinstance(level, int):
            # attempt to get the level if passed a string
            try:
                level = getattr(messages.constants, level.upper())
            except AttributeError:
                levels = messages.constants.DEFAULT_TAGS.values()
                levels_repr = ', '.join('`%s`' % l for l in levels)
                raise ValueError('Bad message level string: `%s`. '
                                 'Possible values are: %s' % (level, levels_repr))

        messages.add_message(request, level, message, extra_tags=extra_tags,
                             fail_silently=fail_silently)

    def get_detail_icon(self, obj):
        opts = self.model._meta
        app_label = opts.app_label.lower()
        object_name = opts.object_name.lower()
        return """
            <a class='iframe' href=%(reverse_url)s?_popup=1&nobuttons=1>
                <img src='%(url)sjmb/images/search.png' alt='%(window_title)s' title='%(window_title)s'/>
            </a>""" % {
            'reverse_url': reverse('%s:%s_%s_detail' % (
                self.admin_site.name, app_label, object_name), args=(obj.pk,)
            ),
            'url': settings.STATIC_URL,
            'window_title': ugt("Detail %s" % object_name)
        }
    get_detail_icon.allow_tags = True
    get_detail_icon.short_description = _("V")

    def get_status_icon(self, obj):
        return obj.status
    get_status_icon.short_description = _("ST")
    get_status_icon.boolean = True
    get_status_icon.admin_order_field = 'status'

    def get_changelist_queryset(self, request):
        """
        Return a queryset that was correctly filtered by any filter (q= and filterset_class)
        """
        # stolen from changelist_view in Django 1.5
        opts = self.model._meta
        app_label = opts.app_label
        if not self.has_change_permission(request, None):
            raise PermissionDenied

        list_display = self.get_list_display(request)
        list_display_links = self.get_list_display_links(request, list_display)
        list_filter = self.get_list_filter(request)

        ChangeList = self.get_changelist(request)
        cl = ChangeList(request, self.model, list_display,
                        list_display_links, list_filter, self.date_hierarchy,
                        self.search_fields, self.list_select_related,
                        self.list_per_page, self.list_max_show_all, self.list_editable,
                        self)
        try:
            return cl.queryset
        except AttributeError:  # pre Django 1.6
            return cl.query_set


admin.site.add_action(actions.export_action)
#admin.site.add_action(actions.export_xls)
